/* Program  inter2d.cc           */
/* By Harald Stauss              */
/* Last change 20.03. 1999       */
/* g++ -o inter2d inter2d.cc -lm */

#include <ctype.h>
#include "tsa.h"
#include "nrc/nrutil.h"
#include "nrc/nrutil.c"
#include "nrc/spline.c"
#include "nrc/splint.c"
#include "nrc/splie2.c"
#include "nrc/splin2.c"

main(int argc, char *argv[]) {
  int           i, lx, ly;
  unsigned long n, n1, n2, nx, ny;
  char          *fin,  *ftin, *fout, *ftout;
  char          *pfin, *pftin, *pfout, *pftout;
  REGIST        r;
  float         *x1a, *x2a, **ya, **y2a, **mya, **my2a;
  float         x1, x2, y, xfac, yfac;
  ofstream      ausdat;


  /*  H e l l o    w o r l d   */

  cout << "inter2d Ver. 1. by H. Stauss, 1999 \n";
  fin=(char *)malloc(255); pfin=fin;
  ftin=(char *)malloc(255); pftin=ftin;
  fout=(char *)malloc(255); pfout=fout;
  ftout=(char *)malloc(255); pftout=ftout;

  /* P a r a m e t e r   e i n l e s e n */

  if (argc != 7)
    {
      cout << "Number of parameters wrong! \n";
      cout << "Two dimensional interpolation \n";
      cout << "inter2d fin fout nx logx ny logy \n";
      cout << "fin    : name of input file (*.asc or *.tsa). \n";
      cout << "fout   : name of output file (*.asc or *.tsa). \n";
      cout << "nx     : number of interpolated values in x direction (cols) \n";
      cout << "         nx=0: same number of values as in input file \n";
      cout << "logx   : 0: linear scale, 1: log scale for x direction \n";
      cout << "ny     : number of interpolated values in y direction (rows) \n";
      cout << "         ny=0: same number of values as in input file \n";
      cout << "logy   : 0: linear scale, 1: log scale for y direction \n";
      goto ende;
    }
  strcpy(fin,argv[1]);
  strcpy(ftin,fin);
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);

  strcpy(fout ,argv[2]);
  strcpy(ftout,fout);
  pftout=strrchr(ftout,'.');
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);
  sscanf(argv[3],"%u",&nx);
  sscanf(argv[4],"%d",&lx);
  sscanf(argv[5],"%u",&ny);
  sscanf(argv[6],"%d",&ly);

  /* E i n g a b e d a t e i e n   o e f f n e n */
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,1);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    cout << "File type for input file not recognized! \n";
    goto ende;
  }
  if (nx==0) nx=r.ch[0].n;
  if (ny==0) ny=r.n;
  cout << "input file : " << fin << "\n";
  cout << "output file: " << fout << "\n";
  cout << "x dim      : " << nx << "\n";
  cout << "y dim      : " << ny << "\n";
  if (lx) cout << "log x scale: yes \n"; else cout << "log x scale: no \n";
  if (ly) cout << "log y scale: yes \n"; else cout << "log y scale: no \n";

  /* Arrays vorbereiten */
  ya =new (float *)[r.n];
  y2a=new (float *)[r.n];
  for (i=0; i<r.n; i++) {
    ya[i] =r.ch[i].data;
    y2a[i]=new (float)[r.ch[0].n];
  }
  mya=submatrix(ya, 0, r.n-1, 0, r.ch[0].n-1, 1, 1);
  my2a=submatrix(y2a, 0, r.n-1, 0, r.ch[0].n-1, 1, 1);
  x1a=new float[r.n];
  for (i=0; i<r.n; i++) x1a[i]=(float)i;
  x2a=new float[r.ch[0].n];
  for (n=0; n<r.ch[0].n; n++) x2a[n]=(float)n;

  /* Spline berechnen */
  splie2(x1a-1, x2a-1, mya, r.n, r.ch[0].n, my2a);

  /* Ausgabedatei ffnen */
  ausdat.open(fout);

  for (n2=0; n2<nx; n2++) {
    if (lx)
      x2=exp(n2 * (log((float)r.ch[0].n-1)/(float)(nx-1)));
    else
      x2=n2*((float)(r.ch[0].n-1)/(float)(nx-1));
    for (n1=0; n1<ny; n1++) {
      if (ly)
	x1=exp(n1 * (log((float)r.n-1)/(float)(ny-1)));
      else
	x1=n1*((float)(r.n-1)/(float)(ny-1));
      splin2(x1a-1, x2a-1, mya, my2a, r.n, r.ch[0].n, x1, x2, &y);
      ausdat << y << "\t";
    }
    ausdat << "\n";
  }

  /* free memory */
  ausdat.close();
  delete[]x1a; delete[]x2a;
  for (i=0; i<r.n; i++) delete[]y2a[i];
  delete[]ya; delete[]y2a; 
  free_submatrix(mya, 1, r.n, 1, r.ch[0].n);
 ende:
  free(fin); free(ftin); free(fout); free(ftout);
  return 0;
} // ende des programms;
